import { Server } from 'socket.io'
import { TSocket } from '../../../../utilities/types'
import { writeFile } from 'node:fs/promises'
import path from 'path'
import fs from 'fs/promises'
import prisma from '../../../../utilities/prisma'

interface IObjectData {
  [key: string]: Buffer
}

/**
 * Handle game master upload object event
 * @param socket
 * @param io
 */
export default function (socket: TSocket, io: Server) {
  socket.on('gm:object:upload', async (data: IObjectData, callback: (response: boolean) => void) => {
    try {
      if (socket.character?.role !== 'gm') {
        callback(false)
        return
      }

      const public_folder = path.join(process.cwd(), 'public', 'objects')

      // Ensure the folder exists
      await fs.mkdir(public_folder, { recursive: true })

      const uploadPromises = Object.entries(data).map(async ([key, objectData]) => {
        const object = await prisma.object.create({
          data: {
            name: key,
            tags: [],
            originX: 0,
            originY: 0
          }
        })

        const uuid = object.id
        const filename = `${uuid}.png`
        const finalFilePath = path.join(public_folder, filename)
        await writeFile(finalFilePath, objectData)
      })

      await Promise.all(uploadPromises)

      callback(true)
    } catch (error) {
      console.error('Error uploading tile:', error)
      callback(false)
    }
  })
}
